#[cfg(feature = "crossbeam")]
use crossbeam_channel::{unbounded, Receiver};

#[cfg(not(feature = "crossbeam"))]
use std::sync::mpsc::{channel as unbounded, Receiver};

#[cfg_attr(target_os = "linux", path = "linux/mod.rs")]
#[cfg_attr(target_os = "windows", path = "windows/mod.rs")]
#[cfg_attr(target_os = "macos", path = "macos/mod.rs")]
pub mod platform_impl;

#[derive(Debug, Clone, Copy, Default)]
pub struct Bounds {
    pub x: i32,
    pub y: i32,
    pub width: i32,
    pub height: i32,
}

#[derive(Debug, Clone, Copy)]
pub enum UnderlayEvent {
    Attach {
        bounds: Bounds,
        has_access: Option<bool>,
        is_fullscreen: Option<bool>,
    },
    Detach,
    MoveResize(Bounds),
    Focus,
    Blur,
    X11FullscreenEvent {
        is_fullscreen: bool,
    },
}

pub fn focus_target() {
    platform_impl::focus_target();
}

pub fn register<T: Into<String>>(window_title: T) -> Receiver<UnderlayEvent> {
    let (tx, rx) = unbounded();

    platform_impl::init(window_title.into(), tx).expect("Catastrophe");

    rx
}
